@@ -12,6 +12,7 @@ set -e
 #
 # Build the DEB's
 #
+./make-deb.sh build/barry-$1.$2.tar.bz2 ubuntu904 ubuntu904
 ./make-deb.sh build/barry-$1.$2.tar.bz2 ubuntu804 ubuntu804
 ./make-deb.sh build/barry-$1.$2.tar.bz2 ubuntu710 ubuntu710
 
@@ -19,6 +20,7 @@ set -e
 #
 # Build the Fedora RPM's
 #
+./make-user-rpm.sh build/barry-$1.$2.tar.bz2 ../rpm/barry.spec fedora11 f11
 ./make-rpm.sh build/barry-$1.$2.tar.bz2 ../rpm/barry.spec fedora9 fc9
 ./make-rpm.sh build/barry-$1.$2.tar.bz2 ../rpm/barry.spec fedora8 fc8
 ./make-rpm.sh build/barry-$1.$2.tar.bz2 ../rpm/barry.spec fedora7 fc7
