@@ -5,19 +5,19 @@
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  * Author: Chris Toshok (toshok@ximian.com)
  * Author: Armin Bauer (armin.bauer@opensync.org)
- * 
+ *
  */
 
 #include "vformat.h"
@@ -44,24 +44,24 @@ static char *quoted_encode_simple (const unsigned char *string, int len);
 
 /**
  * _helper_is_base64 is helper function to check i a string is "b" or "base64"
- * @param check_string string that should be compared with "b" or "base64" 
+ * @param check_string string that should be compared with "b" or "base64"
  * @return 0 if check_string is not base64  and 1 if it is
  */
 static int _helper_is_base64(const char *check_string)
 {
-	if(!g_ascii_strcasecmp ((char *) check_string, "BASE64") || 
+	if(!g_ascii_strcasecmp ((char *) check_string, "BASE64") ||
 	   !g_ascii_strcasecmp ((char *) check_string, "b") )
 		return (1);
 	return (0);
 }
 
 time_t b_vformat_time_to_unix(const char *inptime)
-{	
+{
 	char *date = NULL;
 	char *time = NULL;
 	char *ftime = NULL;
 	if ((ftime = g_strrstr(inptime, "T"))) {
-		
+
 		date = g_strndup(inptime, ftime - inptime);
 		if (ftime[3] == ':')
 			time = g_strndup(ftime + 1, 8);
@@ -70,7 +70,7 @@ time_t b_vformat_time_to_unix(const char *inptime)
 	} else {
 		date = g_strdup(inptime);
 	}
-	
+
 	struct tm btime;
 	memset(&btime, 0, sizeof(struct tm));
 	btime.tm_isdst = -1;
@@ -84,7 +84,7 @@ time_t b_vformat_time_to_unix(const char *inptime)
 		btime.tm_mon = date[4] * 10 + date[5] - '0' * 11 - 1;
 		btime.tm_mday = date[6] * 10 + date[7] - '0' * 11;
 	}
-		
+
 	if (time && strlen(time) == 8) {
 		//Time
 		btime.tm_hour = time[0] * 10 + time[1] - '0' * 11;
@@ -95,7 +95,7 @@ time_t b_vformat_time_to_unix(const char *inptime)
 		btime.tm_min = time[2] * 10 + time[3] - '0' * 11;
 		btime.tm_sec = time[4] * 10 + time[5] - '0' * 11;
 	}
-	
+
 	time_t utime = mktime(&btime);
 	return utime;
 }
@@ -108,30 +108,30 @@ static char *_fold_lines (char *buf)
 	char *next, *next2, *q;
 	gboolean newline = TRUE;
 	gboolean quotedprintable = FALSE;
-	
-	/* 
+
+	/*
 	 *  We're pretty liberal with line folding here. We handle
-	 *  lines folded with \r\n<WS>, \n\r<WS>, \n<WS>, =\r\n and =\n\r. 
+	 *  lines folded with \r\n<WS>, \n\r<WS>, \n<WS>, =\r\n and =\n\r.
 	 *  We also turn single \r's and \n's not followed by <WS> into \r\n's.
 	 */
-	
+
 	while (*p) {
 
 		/* search new lines for quoted printable encoding */
 		if (newline) {
 			for (q=p; *q != '\n' && *q != '\0'; q++)
 				line = g_string_append_unichar (line, g_utf8_get_char (q));
-		
+
 			if (strstr(line->str, "ENCODING=QUOTED-PRINTABLE"))
 				quotedprintable = TRUE;
-			
+
 			g_string_free(line, TRUE);
 			line = g_string_new ("");
 
 			newline = FALSE;
 		}
 
-				
+
 		if ((quotedprintable && *p == '=') || *p == '\r' || *p == '\n') {
 			next = g_utf8_next_char (p);
 			if (*next == '\n' || *next == '\r') {
@@ -149,7 +149,7 @@ static char *_fold_lines (char *buf)
 			else if (*p == '=') {
 				str = g_string_append_unichar (str, g_utf8_get_char (p));
 				p = g_utf8_next_char (p);
-			}	
+			}
 			else if (*next == ' ' || *next == '\t') {
 				p = g_utf8_next_char (next);
 			}
@@ -196,7 +196,7 @@ static void _skip_until (char **p, char *s)
 	char *lp;
 
 	lp = *p;
-	
+
 	while (*lp != '\r' && *lp != '\0') {
 		gboolean s_matches = FALSE;
 		char *ls;
@@ -221,7 +221,7 @@ static void _read_attribute_value_add (b_VFormatAttribute *attr, GString *str, G
 	if (str->len == 0) {
 		b_vformat_attribute_add_value(attr, str->str);
 		return;
-	}      	
+	}
 
 	char *inbuf, *outbuf, *p;
 	size_t inbytesleft, outbytesleft;
@@ -251,7 +251,7 @@ static void _read_attribute_value_add (b_VFormatAttribute *attr, GString *str, G
                         b_vformat_attribute_add_value(attr, str->str);
 
                 }
-	
+
 		iconv_close(cd);
 
 	} else {
@@ -259,7 +259,7 @@ static void _read_attribute_value_add (b_VFormatAttribute *attr, GString *str, G
 		/* no CHARSET was given, if inbuf is already UTF-8 we add str->str */
 		if (g_utf8_validate (inbuf, -1, NULL)) {
 
-			b_vformat_attribute_add_value (attr, str->str);	
+			b_vformat_attribute_add_value (attr, str->str);
 
                 } else {
 
@@ -278,7 +278,7 @@ static void _read_attribute_value_add (b_VFormatAttribute *attr, GString *str, G
                                 b_vformat_attribute_add_value (attr, str->str);
 
                         }
-		
+
 			iconv_close(cd);
 
 		}
@@ -299,10 +299,10 @@ static void _read_attribute_value (b_VFormatAttribute *attr, char **p, int forma
 	while (*lp != '\r' && *lp != '\0') {
 		if (*lp == '=' && format_encoding == VF_ENCODING_QP) {
 			char a, b, x1=0, x2=0;
-		
+
 			if ((a = *(++lp)) == '\0') break;
 			if ((b = *(++lp)) == '\0') break;
-			
+
 			if (isalnum(a)) {
 				if (isalnum(b)) {
 					/* e.g. ...N=C3=BCrnberg\r\n
@@ -321,16 +321,16 @@ static void _read_attribute_value (b_VFormatAttribute *attr, char **p, int forma
 					if (*(++tmplp) == '\r' && *(++tmplp) == '\n' && isalnum(*(++tmplp))) {
 						x1 = a;
 						x2 = *tmplp;
-						lp = tmplp;	
-					}	
+						lp = tmplp;
+					}
 				}
 				else {
 					/* append malformed input, and
 				   	   continue parsing */
 					str = g_string_append_c(str, a);
 					str = g_string_append_c(str, b);
-				}	
-			}	
+				}
+			}
 			else if (a == '=') {
 				char *tmplp = lp;
 				char c, d, e;
@@ -347,7 +347,7 @@ static void _read_attribute_value (b_VFormatAttribute *attr, char **p, int forma
 				   	   continue parsing */
 					str = g_string_append_c(str, a);
 					str = g_string_append_c(str, b);
-				}	
+				}
 			}
 			else {
 				/* append malformed input, and
@@ -363,9 +363,9 @@ static void _read_attribute_value (b_VFormatAttribute *attr, char **p, int forma
 
 				c = (((a>='a'?a-'a'+10:a-'0')&0x0f) << 4)
 					| ((b>='a'?b-'a'+10:b-'0')&0x0f);
-				
+
 				str = g_string_append_c (str, c);
-			}	
+			}
 			lp++;
 			x1 = x2 = 0;
 		}
@@ -438,7 +438,7 @@ static void _read_attribute_params(b_VFormatAttribute *attr, char **p, int *form
 	b_VFormatParam *param = NULL;
 	gboolean in_quote = FALSE;
 	str = g_string_new ("");
-	
+
 	while (*lp != '\0') {
 		if (*lp == '"') {
 			in_quote = !in_quote;
@@ -505,16 +505,16 @@ static void _read_attribute_params(b_VFormatAttribute *attr, char **p, int *form
 						*format_encoding = VF_ENCODING_QP;
 						b_vformat_attribute_param_free (param);
 						param = NULL;
-					} else if ( _helper_is_base64(param->values->data)) { 
+					} else if ( _helper_is_base64(param->values->data)) {
 						*format_encoding = VF_ENCODING_BASE64;
 //						b_vformat_attribute_param_free (param);
 //						param = NULL;
 					}
 				} else if (param && !g_ascii_strcasecmp(param->name, "charset")) {
 					*charset = g_string_new(param->values->data);
-					b_vformat_attribute_param_free (param);	
+					b_vformat_attribute_param_free (param);
 					param = NULL;
-				}	
+				}
 			}
 			else {
 				if (str->len > 0) {
@@ -547,7 +547,7 @@ static void _read_attribute_params(b_VFormatAttribute *attr, char **p, int *form
 				else {
 					/* we've got an attribute with a truly empty
 					   attribute parameter.  So it's of the form:
-					   
+
 					   ATTR;[PARAM=value;]*;[PARAM=value;]*:
 
 					   (note the extra ';')
@@ -590,7 +590,7 @@ static b_VFormatAttribute *_read_attribute (char **p)
 	b_VFormatAttribute *attr = NULL;
 	GString *str, *charset = NULL;
 	char *lp = *p;
-	
+
 	gboolean is_qp = FALSE;
 
 	/* first read in the group/name */
@@ -723,7 +723,7 @@ static void _parse(b_VFormat *evc, const char *str)
 		osync_trace(TRACE_INTERNAL, "invalid utf8 passed to b_VFormat.  Limping along.");
 		*end = '\0';
 	}
-	
+
 	buf = _fold_lines (buf);
 
 	p = buf;
@@ -731,7 +731,7 @@ static void _parse(b_VFormat *evc, const char *str)
 	attr = _read_attribute (&p);
 	if (!attr)
 		attr = _read_attribute (&p);
-	
+
 	if (!attr || attr->group || g_ascii_strcasecmp (attr->name, "begin")) {
 		osync_trace(TRACE_INTERNAL, "vformat began without a BEGIN\n");
 	}
@@ -805,7 +805,7 @@ char *b_vformat_escape_string (const char *s, b_VFormatType type)
 				str = g_string_append_c (str, *p);
 			break;
 		case '\\':
-			/** 
+			/**
 			 * We won't escape backslashes
 			 * on vcard 2.1, unless it is in the end of a value.
 			 * See comments above for a better explanation
@@ -940,10 +940,10 @@ b_VFormatAttribute *b_vformat_find_attribute(b_VFormat *vcard, const char *name,
 					return attr;
 				i++;
 			}
-		}	
+		}
 	}
 	return NULL;
-}	
+}
 
 /*
 b_VFormatAttribute *b_vformat_find_attribute_next(b_VFormatAttribute *last,
@@ -959,7 +959,7 @@ b_VFormatAttribute *b_vformat_find_attribute_next(b_VFormatAttribute *last,
 			if( i == nth )
 				return attr;
 			i++;
-		}	
+		}
 	}
 	return NULL;
 }
@@ -967,7 +967,7 @@ b_VFormatAttribute *b_vformat_find_attribute_next(b_VFormatAttribute *last,
 
 char *b_vformat_to_string (b_VFormat *evc, b_VFormatType type)
 {
-	osync_trace(TRACE_ENTRY, "%s(%p, %i)", __func__, type);
+	osync_trace(TRACE_ENTRY, "%s(%p, %i)", __func__, evc, type);
 	GList *l;
 	GList *v;
 
@@ -1018,7 +1018,7 @@ char *b_vformat_to_string (b_VFormat *evc, b_VFormatType type)
 			/* 5.8.2:
 			 * param        = param-name "=" param-value *("," param-value)
 			 */
-			if( type == VFORMAT_CARD_30 || type == VFORMAT_TODO_20 
+			if( type == VFORMAT_CARD_30 || type == VFORMAT_TODO_20
 			    || type == VFORMAT_EVENT_20) {
 
 				/**
@@ -1036,7 +1036,7 @@ char *b_vformat_to_string (b_VFormat *evc, b_VFormatType type)
 					if (_helper_is_base64((const char *) v->data)) {
 						format_encoding = VF_ENCODING_BASE64;
 						/*Only the "B" encoding of [RFC 2047] is an allowed*/
-						v->data="B";
+						v->data=g_strdup("B");
 					}
 					/**
 					 * QUOTED-PRINTABLE inline encoding has been
@@ -1060,7 +1060,7 @@ char *b_vformat_to_string (b_VFormat *evc, b_VFormatType type)
 				 * have a "TYPE=" parameter
 				**/
 				gboolean must_have_type = FALSE;
-				if (!g_ascii_strcasecmp (attr->name, "PHOTO") || !g_ascii_strcasecmp (attr->name, "LOGO") || !g_ascii_strcasecmp (attr->name, "SOUND") ) 
+				if (!g_ascii_strcasecmp (attr->name, "PHOTO") || !g_ascii_strcasecmp (attr->name, "LOGO") || !g_ascii_strcasecmp (attr->name, "SOUND") )
 					must_have_type = TRUE;
 				if ( must_have_type || g_ascii_strcasecmp (param->name, "TYPE") )
 					attr_str = g_string_append (attr_str, param->name);
@@ -1073,7 +1073,7 @@ char *b_vformat_to_string (b_VFormat *evc, b_VFormatType type)
 					// check for base64 encoding
 					if (_helper_is_base64((const char *) v->data)) {
 						format_encoding = VF_ENCODING_BASE64;
-						v->data="BASE64";
+						v->data=g_strdup("BASE64");
 					}
 					attr_str = g_string_append (attr_str, v->data);
 					if (v->next)
@@ -1088,7 +1088,7 @@ char *b_vformat_to_string (b_VFormat *evc, b_VFormatType type)
 			char *value = v->data;
 			char *escaped_value = NULL;
 
-			if (!g_ascii_strcasecmp (attr->name, "RRULE") && 
+			if (!g_ascii_strcasecmp (attr->name, "RRULE") &&
 				  strstr (value, "BYDAY") == v->data) {
 				attr_str = g_string_append (attr_str, value);
 			} else {
@@ -1150,7 +1150,7 @@ char *b_vformat_to_string (b_VFormat *evc, b_VFormatType type)
 		 * not bytes.  In particular, it would be an error to put a line break
 		 * within a UTF-8 character.
 		*/
-		
+
 		l = 0;
 		do {
 			if (g_utf8_strlen(attr_str->str, attr_str->len) - l > 75) {
@@ -1177,10 +1177,10 @@ char *b_vformat_to_string (b_VFormat *evc, b_VFormatType type)
 		/**
 		 * base64= <MIME RFC 1521 base64 text>
 		 * the end of the text is marked with two CRLF sequences
-		 * this results in one blank line before the start of the 
+		 * this results in one blank line before the start of the
 		 * next property
 		**/
-		if( format_encoding == VF_ENCODING_BASE64 
+		if( format_encoding == VF_ENCODING_BASE64
 		   && (type == VFORMAT_CARD_21))
 			attr_str = g_string_append (attr_str, CRLF);
 
@@ -1207,8 +1207,8 @@ char *b_vformat_to_string (b_VFormat *evc, b_VFormatType type)
 			str = g_string_append (str, "END:VNOTE\r\n");
 			break;
 	}
-	
-	osync_trace(TRACE_EXIT, "%s(%p, %i)", __func__, type);
+
+	osync_trace(TRACE_EXIT, "%s", __func__);
 	return g_string_free (str, FALSE);
 }
 
@@ -1391,10 +1391,10 @@ b_vformat_attribute_add_value_decoded (b_VFormatAttribute *attr, const char *val
 		case VF_ENCODING_BASE64: {
 			char *b64_data = base64_encode_simple (value, len);
 			GString *decoded = g_string_new_len (value, len);
-	
+
 			/* make sure the decoded list is up to date */
 			b_vformat_attribute_get_values_decoded (attr);
-	
+
 			attr->values = g_list_append (attr->values, b64_data);
 			attr->decoded_values = g_list_append (attr->decoded_values, decoded);
 			break;
@@ -1402,10 +1402,10 @@ b_vformat_attribute_add_value_decoded (b_VFormatAttribute *attr, const char *val
 		case VF_ENCODING_QP: {
 			char *qp_data = quoted_encode_simple ((unsigned char*)value, len);
 			GString *decoded = g_string_new (value);
-	
+
 			/* make sure the decoded list is up to date */
 			b_vformat_attribute_get_values_decoded (attr);
-	
+
 			attr->values = g_list_append (attr->values, qp_data);
 			attr->decoded_values = g_list_append (attr->decoded_values, decoded);
 			break;
@@ -1413,10 +1413,10 @@ b_vformat_attribute_add_value_decoded (b_VFormatAttribute *attr, const char *val
 		case VF_ENCODING_8BIT: {
 			char *data = g_strdup(value);
 			GString *decoded = g_string_new (value);
-	
+
 			/* make sure the decoded list is up to date */
 			b_vformat_attribute_get_values_decoded (attr);
-	
+
 			attr->values = g_list_append (attr->values, data);
 			attr->decoded_values = g_list_append (attr->decoded_values, decoded);
 			break;
@@ -1550,14 +1550,18 @@ b_vformat_attribute_add_param (b_VFormatAttribute *attr,
 	}
 }
 
-b_VFormatParam *b_vformat_attribute_find_param(b_VFormatAttribute *attr, const char *name)
+b_VFormatParam *b_vformat_attribute_find_param(b_VFormatAttribute *attr, const char *name, int level)
 {
 	g_return_val_if_fail (attr != NULL, NULL);
 	GList *p = NULL;
 	for (p = attr->params; p; p = p->next) {
 		b_VFormatParam *param = p->data;
-		if (!g_ascii_strcasecmp (param->name, name))
-			return param;
+		if (!g_ascii_strcasecmp (param->name, name)) {
+			if( level == 0 )
+				return param;
+			else
+				level--;
+		}
 	}
 	return NULL;
 }
@@ -1567,7 +1571,7 @@ b_vformat_attribute_set_value (b_VFormatAttribute *attr,
 				int nth, const char *value)
 {
 	GList *param = g_list_nth(attr->values, nth);
-	g_free(param->data);	
+	g_free(param->data);
 	param->data = g_strdup(value);
 }
 
@@ -1603,10 +1607,10 @@ b_vformat_attribute_add_param_with_value (b_VFormatAttribute *attr, const char *
 {
 	g_return_if_fail (attr != NULL);
 	g_return_if_fail (name != NULL);
-	
+
 	if (!value)
 		return;
-	
+
 	b_VFormatParam *param = b_vformat_attribute_param_new(name);
 
 	b_vformat_attribute_param_add_value (param, value);
@@ -1774,14 +1778,14 @@ const char *b_vformat_attribute_get_nth_value(b_VFormatAttribute *attr, int nth)
 	GString *retstr = (GString *)g_list_nth_data(values, nth);
 	if (!retstr)
 		return NULL;
-	
+
 	if (!g_utf8_validate(retstr->str, -1, NULL)) {
 		values = b_vformat_attribute_get_values(attr);
 		if (!values)
 			return NULL;
 		return g_list_nth_data(values, nth);
 	}
-	
+
 	return retstr->str;
 }
 
@@ -1818,7 +1822,7 @@ gboolean b_vformat_attribute_has_param(b_VFormatAttribute *attr, const char *nam
 {
 	g_return_val_if_fail (attr != NULL, FALSE);
 	g_return_val_if_fail (name != NULL, FALSE);
-	
+
 	GList *params = b_vformat_attribute_get_params(attr);
 	GList *p;
 	for (p = params; p; p = p->next) {
@@ -1941,7 +1945,7 @@ static size_t base64_encode_step(unsigned char *in, size_t len, gboolean break_l
 		case 2:	c1 = ((unsigned char *)save)[1];
 			c2 = ((unsigned char *)save)[2]; goto skip2;
 		}
-		
+
 		/* yes, we jump into the loop, no i'm not going to change it, it's beautiful! */
 		while (inptr < inend) {
 			c1 = *inptr++;
@@ -1997,7 +2001,7 @@ static size_t base64_decode_step(unsigned char *in, size_t len, unsigned char *o
 {
 	unsigned char base64_rank[256];
 	base64_init((char*)base64_rank);
-	
+
 	register unsigned char *inptr, *outptr;
 	unsigned char *inend, c;
 	register unsigned int v;
@@ -2072,7 +2076,7 @@ static size_t base64_decode_simple (char *data, size_t len)
 static char *quoted_encode_simple(const unsigned char *string, int len)
 {
 	GString *tmp = g_string_new("");
-	
+
 	int i = 0;
 	while(string[i] != 0) {
 		if (string[i] > 127 || string[i] == 13 || string[i] == 10 || string[i] == '=') {
@@ -2082,7 +2086,7 @@ static char *quoted_encode_simple(const unsigned char *string, int len)
 		}
 		i++;
 	}
-	
+
 	char *ret = tmp->str;
 	g_string_free(tmp, FALSE);
 	return ret;
@@ -2105,17 +2109,17 @@ static size_t quoted_decode_simple (char *data, size_t len)
 		int i = strcspn(string->str, "=");
 		if (i >= strlen(string->str))
 			break;
-		
+
 		strcpy(hex, "0x");
 		strncat(hex, &string->str[i + 1], 2);
 		char rep = ((int)(strtod(hex, NULL)));
 		g_string_erase(string, i, 2);
 		g_string_insert_c(string, i, rep);
 	}
-	
+
 	memset(data, 0, strlen(data));
 	strcpy(data, string->str);
 	g_string_free(string, 1);
-	
+
 	return strlen(data);
 }
