@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2008, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,17 +28,26 @@
 #define VENDOR_RIM		0x0fca
 #define PRODUCT_RIM_BLACKBERRY	0x0001
 #define PRODUCT_RIM_PEARL_DUAL	0x0004
+#define PRODUCT_RIM_PEARL_FLIP	0x8001		// 8200 series
 #define PRODUCT_RIM_PEARL_8120	0x8004
 #define PRODUCT_RIM_PEARL	0x0006
+#define PRODUCT_RIM_STORM	0x8007
 
+#define BLACKBERRY_INTERFACE		0
 #define BLACKBERRY_CONFIGURATION	1
 #define BLACKBERRY_DB_CLASS		0xff
 
+#define IPRODUCT_RIM_HANDHELD		2
+#define IPRODUCT_RIM_MASS_STORAGE	4
+#define IPRODUCT_RIM_COMPOSITE		5
+
 // minimum number of password tries remaining at which Barry gives up
 // for safety
 #define BARRY_MIN_PASSWORD_TRIES	3
 #define BARRY_MIN_PASSWORD_TRIES_ASC	"3"
 
+#define BLACKBERRY_CHARSET "WINDOWS-1252"
+
 namespace Barry {
 
 /// See also the LogLock class.
