@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2008, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,6 +27,14 @@ using namespace std;
 
 namespace Barry {
 
+std::string BadSize::GetMsg(const char *msg, unsigned int d, unsigned int r)
+{
+	std::ostringstream oss;
+	oss << msg << ": Bad packet size, not enough data: DataSize(): " << d
+	    << ". Required size: " << r;
+	return oss.str();
+}
+
 std::string BadSize::GetMsg(unsigned int p, unsigned int d, unsigned int r)
 {
 	std::ostringstream oss;
