@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2008, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 
 #include "ldif.h"
 #include "record.h"
+#include "r_contact.h"
 #include "base64.h"
 #include <stdexcept>
 #include <iostream>
@@ -57,28 +58,30 @@ const ContactLdif::NameToFunc ContactLdif::FieldMap[] = {
 		&ContactLdif::Company, &ContactLdif::SetCompany },
 	{ "DefaultCommunicationsMethod", "Default communications method",
 		&ContactLdif::DefaultCommunicationsMethod, &ContactLdif::SetDefaultCommunicationsMethod },
-	{ "Address1", "Address, line 1",
-		&ContactLdif::Address1, &ContactLdif::SetAddress1 },
-	{ "Address2", "Address, line 2",
-		&ContactLdif::Address2, &ContactLdif::SetAddress2 },
-	{ "Address3", "Address, line 3",
-		&ContactLdif::Address3, &ContactLdif::SetAddress3 },
-	{ "City", "City",
-		&ContactLdif::City, &ContactLdif::SetCity },
-	{ "Province", "Province / State",
-		&ContactLdif::Province, &ContactLdif::SetProvince },
-	{ "PostalCode", "Postal / ZIP code",
-		&ContactLdif::PostalCode, &ContactLdif::SetPostalCode },
-	{ "Country", "Country",
-		&ContactLdif::Country, &ContactLdif::SetCountry },
+	{ "WorkAddress1", "Work Address, line 1",
+		&ContactLdif::WorkAddress1, &ContactLdif::SetWorkAddress1 },
+	{ "WorkAddress2", "Work Address, line 2",
+		&ContactLdif::WorkAddress2, &ContactLdif::SetWorkAddress2 },
+	{ "WorkAddress3", "Work Address, line 3",
+		&ContactLdif::WorkAddress3, &ContactLdif::SetWorkAddress3 },
+	{ "WorkCity", "WorkCity",
+		&ContactLdif::WorkCity, &ContactLdif::SetWorkCity },
+	{ "WorkProvince", "WorkProvince / State",
+		&ContactLdif::WorkProvince, &ContactLdif::SetWorkProvince },
+	{ "WorkPostalCode", "Work Postal / ZIP code",
+		&ContactLdif::WorkPostalCode, &ContactLdif::SetWorkPostalCode },
+	{ "WorkCountry", "WorkCountry",
+		&ContactLdif::WorkCountry, &ContactLdif::SetWorkCountry },
 	{ "JobTitle", "Job Title",
 		&ContactLdif::JobTitle, &ContactLdif::SetJobTitle },
 	{ "PublicKey", "Public key",
 		&ContactLdif::PublicKey, &ContactLdif::SetPublicKey },
 	{ "Notes", "Notes",
 		&ContactLdif::Notes, &ContactLdif::SetNotes },
-	{ "PostalAddress", "Mailing address (includes address lines, city, province, country, and postal code)",
-		&ContactLdif::PostalAddress, &ContactLdif::SetPostalAddress },
+	{ "WorkPostalAddress", "Mailing Work address (includes address lines, city, province, country, and postal code)",
+		&ContactLdif::WorkPostalAddress, &ContactLdif::SetWorkPostalAddress },
+	{ "HomePostalAddress", "Mailing home address (includes address lines, city, province, country, and postal code)",
+		&ContactLdif::HomePostalAddress, &ContactLdif::SetHomePostalAddress },
 	{ "FullName", "First + Last names",
 		&ContactLdif::FullName, &ContactLdif::SetFullName },
 	{ "FQDN", "Fully qualified domain name",
@@ -121,11 +124,11 @@ ContactLdif::ContactLdif(const std::string &baseDN)
 	Map("homePhone", &ContactLdif::HomePhone, &ContactLdif::SetHomePhone);
 	Map("mobile", &ContactLdif::MobilePhone, &ContactLdif::SetMobilePhone);
 	Map("pager", &ContactLdif::Pager, &ContactLdif::SetPager);
-	Map("l", &ContactLdif::City, &ContactLdif::SetCity);
-	Map("st", &ContactLdif::Province, &ContactLdif::SetProvince);
-	Map("postalCode", &ContactLdif::PostalCode, &ContactLdif::SetPostalCode);
+	Map("l", &ContactLdif::WorkCity, &ContactLdif::SetWorkCity);
+	Map("st", &ContactLdif::WorkProvince, &ContactLdif::SetWorkProvince);
+	Map("postalCode", &ContactLdif::WorkPostalCode, &ContactLdif::SetWorkPostalCode);
 	Map("o", &ContactLdif::Company, &ContactLdif::SetCompany);
-	Map("c", &ContactLdif::Country, &ContactLdif::SetCountry);
+	Map("c", &ContactLdif::WorkCountry, &ContactLdif::SetWorkCountry);
 	SetObjectClass("c", "country");
 
 	Map("title", &ContactLdif::JobTitle, &ContactLdif::SetJobTitle);
@@ -134,8 +137,9 @@ ContactLdif::ContactLdif(const std::string &baseDN)
 	Map("cn", &ContactLdif::FullName, &ContactLdif::SetFullName);
 	Map("sn", &ContactLdif::LastName, &ContactLdif::SetLastName);
 	Map("givenName", &ContactLdif::FirstName, &ContactLdif::SetFirstName);
-	Map("street", &ContactLdif::Address1, &ContactLdif::SetAddress1);
-	Map("postalAddress", &ContactLdif::PostalAddress, &ContactLdif::SetPostalAddress);
+	Map("street", &ContactLdif::WorkAddress1, &ContactLdif::SetWorkAddress1);
+	Map("postalAddress", &ContactLdif::WorkPostalAddress, &ContactLdif::SetWorkPostalAddress);
+	Map("homePostalAddress", &ContactLdif::HomePostalAddress, &ContactLdif::SetHomePostalAddress);
 	Map("note", &ContactLdif::Notes, &ContactLdif::SetNotes);
 
 	// add heuristics hooks
@@ -338,37 +342,37 @@ std::string ContactLdif::DefaultCommunicationsMethod(const Barry::Contact &con)
 	return con.DefaultCommunicationsMethod;
 }
 
-std::string ContactLdif::Address1(const Barry::Contact &con) const
+std::string ContactLdif::WorkAddress1(const Barry::Contact &con) const
 {
 	return con.WorkAddress.Address1;
 }
 
-std::string ContactLdif::Address2(const Barry::Contact &con) const
+std::string ContactLdif::WorkAddress2(const Barry::Contact &con) const
 {
 	return con.WorkAddress.Address2;
 }
 
-std::string ContactLdif::Address3(const Barry::Contact &con) const
+std::string ContactLdif::WorkAddress3(const Barry::Contact &con) const
 {
 	return con.WorkAddress.Address3;
 }
 
-std::string ContactLdif::City(const Barry::Contact &con) const
+std::string ContactLdif::WorkCity(const Barry::Contact &con) const
 {
 	return con.WorkAddress.City;
 }
 
-std::string ContactLdif::Province(const Barry::Contact &con) const
+std::string ContactLdif::WorkProvince(const Barry::Contact &con) const
 {
 	return con.WorkAddress.Province;
 }
 
-std::string ContactLdif::PostalCode(const Barry::Contact &con) const
+std::string ContactLdif::WorkPostalCode(const Barry::Contact &con) const
 {
 	return con.WorkAddress.PostalCode;
 }
 
-std::string ContactLdif::Country(const Barry::Contact &con) const
+std::string ContactLdif::WorkCountry(const Barry::Contact &con) const
 {
 	return con.WorkAddress.Country;
 }
@@ -388,11 +392,16 @@ std::string ContactLdif::Notes(const Barry::Contact &con) const
 	return con.Notes;
 }
 
-std::string ContactLdif::PostalAddress(const Barry::Contact &con) const
+std::string ContactLdif::WorkPostalAddress(const Barry::Contact &con) const
 {
 	return con.WorkAddress.GetLabel();
 }
 
+std::string ContactLdif::HomePostalAddress(const Barry::Contact &con) const
+{
+	return con.HomeAddress.GetLabel();
+}
+
 std::string ContactLdif::FullName(const Barry::Contact &con) const
 {
 	return con.GetFullName();
@@ -489,37 +498,37 @@ void ContactLdif::SetDefaultCommunicationsMethod(Barry::Contact &con, const std:
 	con.DefaultCommunicationsMethod = val;
 }
 
-void ContactLdif::SetAddress1(Barry::Contact &con, const std::string &val) const
+void ContactLdif::SetWorkAddress1(Barry::Contact &con, const std::string &val) const
 {
 	con.WorkAddress.Address1 = val;
 }
 
-void ContactLdif::SetAddress2(Barry::Contact &con, const std::string &val) const
+void ContactLdif::SetWorkAddress2(Barry::Contact &con, const std::string &val) const
 {
 	con.WorkAddress.Address2 = val;
 }
 
-void ContactLdif::SetAddress3(Barry::Contact &con, const std::string &val) const
+void ContactLdif::SetWorkAddress3(Barry::Contact &con, const std::string &val) const
 {
 	con.WorkAddress.Address3 = val;
 }
 
-void ContactLdif::SetCity(Barry::Contact &con, const std::string &val) const
+void ContactLdif::SetWorkCity(Barry::Contact &con, const std::string &val) const
 {
 	con.WorkAddress.City = val;
 }
 
-void ContactLdif::SetProvince(Barry::Contact &con, const std::string &val) const
+void ContactLdif::SetWorkProvince(Barry::Contact &con, const std::string &val) const
 {
 	con.WorkAddress.Province = val;
 }
 
-void ContactLdif::SetPostalCode(Barry::Contact &con, const std::string &val) const
+void ContactLdif::SetWorkPostalCode(Barry::Contact &con, const std::string &val) const
 {
 	con.WorkAddress.PostalCode = val;
 }
 
-void ContactLdif::SetCountry(Barry::Contact &con, const std::string &val) const
+void ContactLdif::SetWorkCountry(Barry::Contact &con, const std::string &val) const
 {
 	con.WorkAddress.Country = val;
 }
@@ -539,11 +548,18 @@ void ContactLdif::SetNotes(Barry::Contact &con, const std::string &val) const
 	con.Notes = val;
 }
 
-void ContactLdif::SetPostalAddress(Barry::Contact &con, const std::string &val) const
+void ContactLdif::SetWorkPostalAddress(Barry::Contact &con, const std::string &val) const
 {
-	// fixme;
-//	throw std::runtime_error("SetPostalAddress() not implemented");
-//	std::cout << "SetPostalAddress() not implemented: " << val << std::endl;
+	// FIXME;
+//	throw std::runtime_error("SetWorkPostalAddress() not implemented");
+//	std::cout << "SetWorkPostalAddress() not implemented: " << val << std::endl;
+}
+
+void ContactLdif::SetHomePostalAddress(Barry::Contact &con, const std::string &val) const
+{
+	// FIXME;
+//	throw std::runtime_error("SetHomePostalAddress() not implemented");
+//	std::cout << "SetHomePostalAddress() not implemented: " << val << std::endl;
 }
 
 void ContactLdif::SetFullName(Barry::Contact &con, const std::string &val) const
