@@ -12,7 +12,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2008, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -64,6 +64,36 @@ struct MessageAddress				// used for Message records
 } __attribute__ ((packed));
 
 
+///////////////////////////////////////////////////////////////////////////////
+// SMS Message field and record structures
+
+struct SMSMetaData
+{
+	uint8_t		recv; // if received, this is set to 1; otherwise 0
+	uint8_t		flags;
+#define SMS_FLG_NEW_CONVERSATION 0x20
+#define SMS_FLG_SAVED 0x10
+#define SMS_FLG_DELETED 0x08
+#define SMS_FLG_OPENED 0x01
+
+	uint8_t		new_flag;
+	uint16_t	zero; // constantly 0
+	uint32_t	status;
+#define SMS_STA_RECEIVED 0x000007ff
+#define SMS_STA_DRAFT 0x7fffffff
+
+	uint32_t	error_id;
+	uint64_t	timestamp;
+	uint64_t	service_center_timestamp;
+	uint8_t		dcs;
+#define SMS_DCS_7BIT 0x00
+#define SMS_DCS_8BIT 0x01
+#define SMS_DCS_UCS2 0x02
+
+} __attribute__ ((packed));
+#define SMS_METADATA_SIZE	(sizeof(Barry::Protocol::SMSMetaData))
+
+
 
 ///////////////////////////////////////////////////////////////////////////////
 // Record Field Formats
@@ -78,6 +108,8 @@ struct CommonField
 
 		GroupLink	link;
 		MessageAddress	addr;
+		SMSMetaData	sms_metadata;
+		uint64_t	timestamp;
 		uint32_t	uint32;
 		int32_t		min1900;
 		uint16_t	code;
@@ -222,6 +254,80 @@ struct CalendarRecurrenceDataField  // as documented in the Cassis project spec
 } __attribute__ ((packed));
 #define CALENDAR_RECURRENCE_DATA_FIELD_SIZE	sizeof(Barry::Protocol::CalendarRecurrenceDataField)
 
+//
+// Calendar record: field constants
+//
+
+#define CR_FREEBUSY_FREE		0
+#define CR_FREEBUSY_TENTATIVE		1
+#define CR_FREEBUSY_BUSY		2
+#define CR_FREEBUSY_OUT_OF_OFFICE	3
+#define CR_FREEBUSY_RANGE_LOW		0
+#define CR_FREEBUSY_RANGE_HIGH		3
+
+#define CR_CLASS_PUBLIC			0
+#define CR_CLASS_CONFIDENTIAL		1
+#define CR_CLASS_PRIVATE		2
+#define CR_CLASS_RANGE_LOW		0
+#define CR_CLASS_RANGE_HIGH		2
+
+
+//
+// Task record: field constants
+//
+
+#define TR_ALARM_DATE			1
+#define TR_ALARM_RELATIVE		2
+#define TR_ALARM_RANGE_LOW		1
+#define TR_ALARM_RANGE_HIGH		2
+
+#define TR_PRIORITY_HIGH		0
+#define TR_PRIORITY_NORMAL		1
+#define TR_PRIORITY_LOW			2
+#define TR_PRIORITY_RANGE_LOW		0
+#define TR_PRIORITY_RANGE_HIGH		2
+
+#define TR_STATUS_NOT_STARTED		0
+#define TR_STATUS_IN_PROGRESS		1
+#define TR_STATUS_COMPLETED		2
+#define TR_STATUS_WAITING		3
+#define TR_STATUS_DEFERRED		4
+#define TR_STATUS_RANGE_LOW		0
+#define TR_STATUS_RANGE_HIGH		4
+
+//
+// Phone Call Logs record: field constants
+//
+//
+#define CLL_DIRECTION_RECEIVER		0
+#define CLL_DIRECTION_EMITTER		1
+#define CLL_DIRECTION_FAILED		2
+#define CLL_DIRECTION_MISSING		3
+#define CLL_DIRECTION_RANGE_LOW		0
+#define CLL_DIRECTION_RANGE_HIGH	3
+
+#define CLL_PHONETYPE_UNDEFINED		0
+#define CLL_PHONETYPE_OFFICE		1
+#define CLL_PHONETYPE_HOME			2
+#define CLL_PHONETYPE_MOBILE		3
+#define CLL_PHONETYPE_RANGE_LOW		0
+#define CLL_PHONETYPE_RANGE_HIGH	3
+
+//
+// Folder record: field constants
+//
+
+#define FR_TYPE_SUBTREE			0x00
+#define FR_TYPE_DELETED			0x01
+#define FR_TYPE_INBOX			0x02
+#define FR_TYPE_OUTBOX			0x03
+#define FR_TYPE_SENT			0x04
+#define FR_TYPE_OTHER			0x05
+#define FR_TYPE_DRAFT			0x0a
+
+#define FR_STATUS_ORPHAN		0x50
+#define FR_STATUS_UNFILED		0x51
+#define FR_STATUS_FILED			0x52
 
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -342,6 +448,13 @@ struct ModeSelect
 	} __attribute__ ((packed)) response;
 } __attribute__ ((packed));
 
+struct Echo
+{
+	uint64_t	ticks;			// number of microseconds since
+						// host system startup
+} __attribute__ ((packed));
+#define ECHO_COMMAND_SIZE		(sizeof(Barry::Protocol::Echo))
+
 
 ///////////////////////////////////////////////////////////////////////////////
 // Protocol command structures
@@ -360,6 +473,7 @@ struct SocketCommand
 		AttributeFetch		fetch;
 		ModeSelect		mode;
 		uint8_t			raw[1];
+		Echo			echo;
 
 	} __attribute__ ((packed)) u;
 } __attribute__ ((packed));
@@ -444,14 +558,14 @@ struct MessageRecord
 	uint32_t	field7;		// always 0x00000000
 	uint32_t	field8;		// always 0x00000000
 	uint16_t	field9;		// always 0x0000
-	
+
 	uint16_t	dateReceived;	// the first two of these time fields are always the same
-	uint16_t	timeReceived;	// 
+	uint16_t	timeReceived;	//
 	uint16_t	dateDuplicate;	// On mail sent from the BB all three fields are identical
 	uint16_t	timeDuplicate;	// (time sent)
-	uint16_t	dateSent;	
+	uint16_t	dateSent;
 	uint16_t	timeSent;
-	
+
 	uint16_t	priority;	// priority field
 	uint32_t	field14;	// always 0x00000000
 	uint32_t	field15;	// always 0x00000000
@@ -465,18 +579,18 @@ struct MessageRecord
 	uint32_t	inReplyTo;	// reply to message?
 	uint32_t	field22;	// always 0x00000000
 	uint16_t	field23;	// FIXME
-	
+
 	uint32_t	folderOne;	// these are the 'folders' the message is in
 	uint32_t	folderTwo;	//
-	 
+
 	uint16_t	replyMessageFlags;	// 0xfffe on recvd messages
 					// 0x001b on reply
 					// 0x0015 on send
 					// 0x3 pin send
 					// 0x2 on pin recv
 	uint16_t	field27;	// set to 0x00000004 on PIN reply, 0x00000005 otherwise
-	uint32_t	headerUID;	// yet another copy of the UID (RecId)	
-	
+	uint32_t	headerUID;	// yet another copy of the UID (RecId)
+
 	uint32_t	field29;	// always 0x00000000
 	uint16_t	field30;	// always 0x0002
 	uint16_t	field31;	// always 0x00000000
@@ -534,6 +648,96 @@ struct DBAccess
 
 
 
+///////////////////////////////////////////////////////////////////////////////
+// Javaloader protocol structure
+
+struct JLDirEntry
+{
+	uint16_t	unknown;
+	uint32_t	timestamp;
+	uint16_t	filename_size;
+	uint8_t		filename[1];
+	// the rest of the packet is variable length
+	// another string for version, then:
+	// uint32_t	cod_size;
+
+} __attribute__ ((packed));
+#define SB_JLDIRENTRY_HEADER_SIZE		(sizeof(Barry::Protocol::JLDirEntry) - 1)
+
+struct JLCommand
+{
+	uint8_t		command;
+	uint8_t		unknown;	// nearly always 0, might be top half of command
+	uint16_t	size;
+} __attribute__ ((packed));
+#define SB_JLCOMMAND_HEADER_SIZE		(sizeof(Barry::Protocol::JLCommand))
+
+struct JLResponse
+{
+	uint8_t		command;
+	uint8_t		unknown;
+	uint16_t	expect;
+} __attribute__ ((packed));
+#define SB_JLRESPONSE_HEADER_SIZE		(sizeof(Barry::Protocol::JLResponse))
+
+struct JLScreenInfo
+{
+	uint16_t	unknown1;
+	uint16_t	unknown2;
+	uint16_t	unknown3;
+	uint16_t	width;
+	uint16_t	height;
+	uint16_t	unknown4;
+	uint16_t	unknown5;
+	uint16_t	unknown6;
+} __attribute__ ((packed));
+#define SB_JLSCREENINFO_SIZE			(sizeof(Barry::Protocol::JLScreenInfo))
+
+struct JLEventlogEntry
+{
+	uint16_t	size;
+	// remainder of packet is variable
+	// it contains the log data as an ASCII (UTF-8?) string
+} __attribute__ ((packed));
+#define SB_JLEVENTLOG_ENTRY_HEADER_SIZE		(sizeof(Barry::Protocol::JLEventlogEntry))
+
+struct JLDeviceInfo
+{
+	uint32_t	hardware_id;
+	uint32_t	pin;
+	uint32_t	os_version;
+	uint32_t	vm_version;
+	uint32_t	radio_id;
+	uint32_t	vendor_id;
+	uint32_t	active_wafs;
+	// older devices (such as 7130) don't this extra data in the
+	// device info packet and will therefore fail the size check
+	//uint8_t		raw[4];
+} __attribute__ ((packed));
+#define SB_JLDEVICEINFO_SIZE			(sizeof(Barry::Protocol::JLDeviceInfo))
+
+struct JLPacket
+{
+	uint16_t	socket;
+	uint16_t	size;		// total size of data packet
+
+	union PacketData
+	{
+		JLCommand		command;
+		JLResponse		response;
+		JLScreenInfo		screeninfo;
+		JLEventlogEntry		logentry;
+		JLDeviceInfo		devinfo;
+		uint8_t			raw[1];
+		char			filename[1];
+		uint32_t		cod_size;
+		uint32_t		timestamp;
+		uint16_t		id;
+	} __attribute__ ((packed)) u;
+
+} __attribute__ ((packed));
+#define SB_JLPACKET_HEADER_SIZE		(sizeof(Barry::Protocol::JLPacket) - sizeof(Barry::Protocol::JLPacket::PacketData))
+
 ///////////////////////////////////////////////////////////////////////////////
 // Main packet struct
 
@@ -555,13 +759,14 @@ struct Packet
 } __attribute__ ((packed));
 #define SB_PACKET_HEADER_SIZE			(sizeof(Barry::Protocol::Packet) - sizeof(Barry::Protocol::Packet::PacketData))
 
-// minimum required sizes for various responses
-#define MIN_PACKET_SIZE		6
+// WARNING : For JavaLoader we have some packet with 5 size !
+#define MIN_PACKET_SIZE			5
+#define MIN_PACKET_DATA_SIZE		4
 
 
 // maximum sizes
-#define MAX_PACKET_SIZE		0x400	// anything beyond this needs to be
-					// fragmented
+#define MAX_PACKET_SIZE			0x400	// anything beyond this needs to be fragmented
+#define MAX_PACKET_DATA_SIZE		0x7FC	// for data packet (JavaLoader)
 
 /////////////////////////////////////////////////////////////////////////////
 //
@@ -589,7 +794,9 @@ struct Packet
 #define COMMAND(data)				(((const Barry::Protocol::Packet *)data.GetData())->command)
 #define IS_COMMAND(data, cmd)			(COMMAND(data) == cmd)
 #define MAKE_PACKET(var, data)			const Barry::Protocol::Packet *var = (const Barry::Protocol::Packet *) (data).GetData()
+#define MAKE_JLPACKET(var, data)		const Barry::Protocol::JLPacket *var = (const Barry::Protocol::JLPacket *) (data).GetData()
 #define MAKE_PACKETPTR_BUF(var, ptr)		Barry::Protocol::Packet *var = (Barry::Protocol::Packet *)ptr
+#define MAKE_JLPACKETPTR_BUF(var, ptr)		Barry::Protocol::JLPacket *var = (Barry::Protocol::JLPacket *)ptr
 #define MAKE_RECORD(type,var,data,off)		type *var = (type *) ((data).GetData() + (off))
 #define MAKE_RECORD_PTR(type,var,data,off)	type *var = (type *) ((data) + (off))
 
@@ -605,7 +812,7 @@ struct Packet
 
 
 // checks packet size and throws BError if not right
-void CheckSize(const Barry::Data &packet, size_t requiredsize = MIN_PACKET_SIZE);
+void CheckSize(const Barry::Data &packet, size_t requiredsize);
 unsigned int GetSize(const Barry::Data &packet);
 
 }} // namespace Barry::Protocol
