@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2008, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 #include "time.h"
 #include "error.h"
 #include "endian.h"
+#include "iconv.h"
 #include <ostream>
 #include <iomanip>
 #include <time.h>
@@ -92,48 +93,49 @@ namespace Barry {
 #define CFC_IMAGE		77	// 0x4D
 #define CFC_BIRTHDAY		82	// 0x52
 #define CFC_ANNIVERSARY		83	// 0x53
+#define CFC_UNIQUEID		85	// 0x55
 #define CFC_INVALID_FIELD	255
 
 // Contact code to field table
-FieldLink<Contact> ContactFieldLinks[] = {
-   { CFC_PHONE,        "Phone",      0,0,                 &Contact::Phone, 0, 0 },
-   { CFC_FAX,          "Fax",        "facsimileTelephoneNumber",0, &Contact::Fax, 0, 0 },
-   { CFC_WORK_PHONE,   "WorkPhone",  "telephoneNumber",0, &Contact::WorkPhone, 0, 0 },
-   { CFC_HOME_PHONE,   "HomePhone",  "homePhone",0,       &Contact::HomePhone, 0, 0 },
-   { CFC_MOBILE_PHONE, "MobilePhone","mobile",0,          &Contact::MobilePhone, 0, 0 },
-   { CFC_PAGER,        "Pager",      "pager",0,           &Contact::Pager, 0, 0 },
-   { CFC_PIN,          "PIN",        0,0,                 &Contact::PIN, 0, 0 },
-   { CFC_RADIO,        "Radio",      0,0,                 &Contact::Radio, 0, 0 },
-   { CFC_WORK_PHONE_2, "WorkPhone2", 0,0,                 &Contact::WorkPhone2, 0, 0 },
-   { CFC_HOME_PHONE_2, "HomePhone2", 0,0,                 &Contact::HomePhone2, 0, 0 },
-   { CFC_OTHER_PHONE,  "OtherPhone", 0,0,                 &Contact::OtherPhone, 0, 0 },
-   { CFC_COMPANY,      "Company",    "o",0,               &Contact::Company, 0, 0 },
-   { CFC_DEFAULT_COMM_METHOD,"DefaultCommMethod",0,0,     &Contact::DefaultCommunicationsMethod, 0, 0 },
-   { CFC_ADDRESS1,     "WorkAddress1",   0,0,             0, 0, 0, &Contact::WorkAddress, &PostalAddress::Address1 },
-   { CFC_ADDRESS2,     "WorkAddress2",   0,0,             0, 0, 0, &Contact::WorkAddress, &PostalAddress::Address2 },
-   { CFC_ADDRESS3,     "WorkAddress3",   0,0,             0, 0, 0, &Contact::WorkAddress, &PostalAddress::Address3 },
-   { CFC_CITY,         "WorkCity",       "l",0,           0, 0, 0, &Contact::WorkAddress, &PostalAddress::City },
-   { CFC_PROVINCE,     "WorkProvince",   "st",0,          0, 0, 0, &Contact::WorkAddress, &PostalAddress::Province },
-   { CFC_POSTAL_CODE,  "WorkPostalCode", "postalCode",0,  0, 0, 0, &Contact::WorkAddress, &PostalAddress::PostalCode },
-   { CFC_COUNTRY,      "WorkCountry",    "c", "country",  0, 0, 0, &Contact::WorkAddress, &PostalAddress::Country },
-   { CFC_TITLE,        "JobTitle",   "title",0,           &Contact::JobTitle, 0, 0 },
-   { CFC_PUBLIC_KEY,   "PublicKey",  0,0,                 &Contact::PublicKey, 0, 0 },
-   { CFC_URL,          "URL",        0,0,                 &Contact::URL, 0, 0 },
-   { CFC_PREFIX,       "Prefix",     0,0,                 &Contact::Prefix, 0, 0 },
-   { CFC_HOME_ADDRESS1,"HomeAddress1", 0,0,               0, 0, 0, &Contact::HomeAddress, &PostalAddress::Address1, },
-   { CFC_HOME_ADDRESS2,"HomeAddress2", 0,0,               0, 0, 0, &Contact::HomeAddress, &PostalAddress::Address2, },
-   { CFC_HOME_ADDRESS3,"HomeAddress3", 0,0,               0, 0, 0, &Contact::HomeAddress, &PostalAddress::Address3, },
-   { CFC_NOTES,        "Notes",      0,0,                 &Contact::Notes, 0, 0 },
-   { CFC_USER_DEFINED_1, "UserDefined1", 0,0,             &Contact::UserDefined1, 0, 0 },
-   { CFC_USER_DEFINED_2, "UserDefined2", 0,0,             &Contact::UserDefined2, 0, 0 },
-   { CFC_USER_DEFINED_3, "UserDefined3", 0,0,             &Contact::UserDefined3, 0, 0 },
-   { CFC_USER_DEFINED_4, "UserDefined4", 0,0,             &Contact::UserDefined4, 0, 0 },
-   { CFC_HOME_CITY,    "HomeCity",   0,0,                 0, 0, 0, &Contact::HomeAddress, &PostalAddress::City, },
-   { CFC_HOME_PROVINCE,"HomeProvince", 0,0,               0, 0, 0, &Contact::HomeAddress, &PostalAddress::Province, },
-   { CFC_HOME_POSTAL_CODE, "HomePostalCode", 0,0,         0, 0, 0, &Contact::HomeAddress, &PostalAddress::PostalCode, },
-   { CFC_HOME_COUNTRY, "HomeCountry",0,0,                 0, 0, 0, &Contact::HomeAddress, &PostalAddress::Country, },
-   { CFC_IMAGE,        "Image",      0,0,                 &Contact::Image, 0, 0 },
-   { CFC_INVALID_FIELD,"EndOfList",  0, 0, 0 }
+static FieldLink<Contact> ContactFieldLinks[] = {
+   { CFC_PHONE,        "Phone",      0,0,                 &Contact::Phone, 0, 0, 0, 0, true },
+   { CFC_FAX,          "Fax",        "facsimileTelephoneNumber",0, &Contact::Fax, 0, 0, 0, 0, true },
+   { CFC_WORK_PHONE,   "WorkPhone",  "telephoneNumber",0, &Contact::WorkPhone, 0, 0, 0, 0, true },
+   { CFC_HOME_PHONE,   "HomePhone",  "homePhone",0,       &Contact::HomePhone, 0, 0, 0, 0, true },
+   { CFC_MOBILE_PHONE, "MobilePhone","mobile",0,          &Contact::MobilePhone, 0, 0, 0, 0, true },
+   { CFC_PAGER,        "Pager",      "pager",0,           &Contact::Pager, 0, 0, 0, 0, true },
+   { CFC_PIN,          "PIN",        0,0,                 &Contact::PIN, 0, 0, 0, 0, true },
+   { CFC_RADIO,        "Radio",      0,0,                 &Contact::Radio, 0, 0, 0, 0, true },
+   { CFC_WORK_PHONE_2, "WorkPhone2", 0,0,                 &Contact::WorkPhone2, 0, 0, 0, 0, true },
+   { CFC_HOME_PHONE_2, "HomePhone2", 0,0,                 &Contact::HomePhone2, 0, 0, 0, 0, true },
+   { CFC_OTHER_PHONE,  "OtherPhone", 0,0,                 &Contact::OtherPhone, 0, 0, 0, 0, true },
+   { CFC_COMPANY,      "Company",    "o",0,               &Contact::Company, 0, 0, 0, 0, true },
+   { CFC_DEFAULT_COMM_METHOD,"DefaultCommMethod",0,0,     &Contact::DefaultCommunicationsMethod, 0, 0, 0, 0, true },
+   { CFC_ADDRESS1,     "WorkAddress1",   0,0,             0, 0, 0, &Contact::WorkAddress, &PostalAddress::Address1, true },
+   { CFC_ADDRESS2,     "WorkAddress2",   0,0,             0, 0, 0, &Contact::WorkAddress, &PostalAddress::Address2, true },
+   { CFC_ADDRESS3,     "WorkAddress3",   0,0,             0, 0, 0, &Contact::WorkAddress, &PostalAddress::Address3, true },
+   { CFC_CITY,         "WorkCity",       "l",0,           0, 0, 0, &Contact::WorkAddress, &PostalAddress::City, true },
+   { CFC_PROVINCE,     "WorkProvince",   "st",0,          0, 0, 0, &Contact::WorkAddress, &PostalAddress::Province, true },
+   { CFC_POSTAL_CODE,  "WorkPostalCode", "postalCode",0,  0, 0, 0, &Contact::WorkAddress, &PostalAddress::PostalCode, true },
+   { CFC_COUNTRY,      "WorkCountry",    "c", "country",  0, 0, 0, &Contact::WorkAddress, &PostalAddress::Country, true },
+   { CFC_TITLE,        "JobTitle",   "title",0,           &Contact::JobTitle, 0, 0, 0, 0, true },
+   { CFC_PUBLIC_KEY,   "PublicKey",  0,0,                 &Contact::PublicKey, 0, 0, 0, 0, false },
+   { CFC_URL,          "URL",        0,0,                 &Contact::URL, 0, 0, 0, 0, true },
+   { CFC_PREFIX,       "Prefix",     0,0,                 &Contact::Prefix, 0, 0, 0, 0, true },
+   { CFC_HOME_ADDRESS1,"HomeAddress1", 0,0,               0, 0, 0, &Contact::HomeAddress, &PostalAddress::Address1, true },
+   { CFC_HOME_ADDRESS2,"HomeAddress2", 0,0,               0, 0, 0, &Contact::HomeAddress, &PostalAddress::Address2, true },
+   { CFC_HOME_ADDRESS3,"HomeAddress3", 0,0,               0, 0, 0, &Contact::HomeAddress, &PostalAddress::Address3, true },
+   { CFC_NOTES,        "Notes",      0,0,                 &Contact::Notes, 0, 0, 0, 0, true },
+   { CFC_USER_DEFINED_1, "UserDefined1", 0,0,             &Contact::UserDefined1, 0, 0, 0, 0, true },
+   { CFC_USER_DEFINED_2, "UserDefined2", 0,0,             &Contact::UserDefined2, 0, 0, 0, 0, true },
+   { CFC_USER_DEFINED_3, "UserDefined3", 0,0,             &Contact::UserDefined3, 0, 0, 0, 0, true },
+   { CFC_USER_DEFINED_4, "UserDefined4", 0,0,             &Contact::UserDefined4, 0, 0, 0, 0, true },
+   { CFC_HOME_CITY,    "HomeCity",   0,0,                 0, 0, 0, &Contact::HomeAddress, &PostalAddress::City, true },
+   { CFC_HOME_PROVINCE,"HomeProvince", 0,0,               0, 0, 0, &Contact::HomeAddress, &PostalAddress::Province, true },
+   { CFC_HOME_POSTAL_CODE, "HomePostalCode", 0,0,         0, 0, 0, &Contact::HomeAddress, &PostalAddress::PostalCode, true },
+   { CFC_HOME_COUNTRY, "HomeCountry",0,0,                 0, 0, 0, &Contact::HomeAddress, &PostalAddress::Country, true },
+   { CFC_IMAGE,        "Image",      0,0,                 &Contact::Image, 0, 0, 0, 0, false },
+   { CFC_INVALID_FIELD,"EndOfList",  0, 0, 0, 0, 0, 0, 0, false }
 };
 
 Contact::Contact()
@@ -148,7 +150,8 @@ Contact::~Contact()
 }
 
 const unsigned char* Contact::ParseField(const unsigned char *begin,
-					 const unsigned char *end)
+					 const unsigned char *end,
+					 const IConverter *ic)
 {
 	const CommonField *field = (const CommonField *) begin;
 
@@ -169,11 +172,15 @@ const unsigned char* Contact::ParseField(const unsigned char *begin,
 			if( b->strMember ) {
 				std::string &s = this->*(b->strMember);
 				s = ParseFieldString(field);
+				if( b->iconvNeeded && ic )
+					s = ic->FromBB(s);
 				return begin;	// done!
 			}
 			else if( b->postMember && b->postField ) {
 				std::string &s = (this->*(b->postMember)).*(b->postField);
 				s = ParseFieldString(field);
+				if( b->iconvNeeded && ic )
+					s = ic->FromBB(s);
 				return begin;
 			}
 			else {
@@ -186,7 +193,10 @@ const unsigned char* Contact::ParseField(const unsigned char *begin,
 	switch( field->type )
 	{
 	case CFC_EMAIL: {
-		EmailAddresses.push_back( ParseFieldString(field) );
+		std::string s = ParseFieldString(field);
+		if( ic )
+			s = ic->FromBB(s);
+		EmailAddresses.push_back( s );
 		}
 		return begin;
 
@@ -204,6 +214,8 @@ const unsigned char* Contact::ParseField(const unsigned char *begin,
 		}
 
 		*name = ParseFieldString(field);
+		if( ic )
+			*name = ic->FromBB(*name);
 		}
 		return begin;
 
@@ -221,7 +233,9 @@ const unsigned char* Contact::ParseField(const unsigned char *begin,
 
 	case CFC_CATEGORY: {
 		std::string catstring = ParseFieldString(field);
-		CategoryStr2List(catstring, Categories);
+		if( ic )
+			catstring = ic->FromBB(catstring);
+		Categories.CategoryStr2List(catstring);
 		}
 		return begin;
 
@@ -254,10 +268,10 @@ void Contact::ParseHeader(const Data &data, size_t &offset)
 }
 
 // this is called by the RecordParser<> class, which checks size for us
-void Contact::ParseFields(const Data &data, size_t &offset)
+void Contact::ParseFields(const Data &data, size_t &offset, const IConverter *ic)
 {
 	const unsigned char *finish = ParseCommonFields(*this,
-		data.GetData() + offset, data.GetData() + data.GetSize());
+		data.GetData() + offset, data.GetData() + data.GetSize(), ic);
 	offset += finish - (data.GetData() + offset);
 }
 
@@ -271,7 +285,7 @@ void Contact::BuildHeader(Data &data, size_t &offset) const
 //
 /// Build fields part of record
 ///
-void Contact::BuildFields(Data &data, size_t &offset) const
+void Contact::BuildFields(Data &data, size_t &offset, const IConverter *ic) const
 {
 	data.Zap();
 
@@ -286,23 +300,37 @@ void Contact::BuildFields(Data &data, size_t &offset) const
 		BuildField(data, offset, CFC_GROUP_FLAG, 'G');
 
 	// special fields not in type table
-	if( FirstName.size() )
-		BuildField(data, offset, CFC_NAME, FirstName);
+	if( FirstName.size() ) {
+		std::string s = ic ? ic->ToBB(FirstName) : FirstName;
+		BuildField(data, offset, CFC_NAME, s);
+	}
 	if( LastName.size() ) {
 		if( !FirstName.size() ) {
 			// order matters with first/last name, and if
 			// last name exists, and first name doesn't,
 			// insert blank first name ahead of it
-			BuildField(data, offset, CFC_NAME, FirstName);
+			BuildField(data, offset, CFC_NAME, "");
 		}
-		BuildField(data, offset, CFC_NAME, LastName);
+		BuildField(data, offset, CFC_NAME, ic ? ic->ToBB(LastName) : LastName);
 	}
 
+//	FIXME
+//	// add unknown data
+//	char buffer[] = { 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff };
+//	BuildField(data, offset, 0x54, buffer, 8);
+
+	// With the BlackBerry Storm, I have to add this entry.
+	// Otherwise the uniqueId of this contact is reseted !
+	// The device seems accept the multiple contact with the same uniqueId,
+	// but the synchronization process uses this uniqueId to identify the contact.
+	// add uniqueId
+	BuildField(data, offset, CFC_UNIQUEID, RecordId);
+
 	// add all email addresses
 	EmailList::const_iterator eai = EmailAddresses.begin();
 	for( ; eai != EmailAddresses.end(); ++eai ) {
 		if( eai->size() ) {
-			BuildField(data, offset, CFC_EMAIL, *eai);
+			BuildField(data, offset, CFC_EMAIL, ic ? ic->ToBB(*eai) : *eai);
 		}
 	}
 
@@ -315,13 +343,15 @@ void Contact::BuildFields(Data &data, size_t &offset) const
 		if( b->strMember ) {
 			const std::string &field = this->*(b->strMember);
 			if( field.size() ) {
-				BuildField(data, offset, b->type, field);
+				std::string s = (b->iconvNeeded && ic) ? ic->ToBB(field) : field;
+				BuildField(data, offset, b->type, s);
 			}
 		}
 		else if( b->postMember && b->postField ) {
 			const std::string &field = (this->*(b->postMember)).*(b->postField);
 			if( field.size() ) {
-				BuildField(data, offset, b->type, field);
+				std::string s = (b->iconvNeeded && ic) ? ic->ToBB(field) : field;
+				BuildField(data, offset, b->type, s);
 			}
 		}
 	}
@@ -339,8 +369,8 @@ void Contact::BuildFields(Data &data, size_t &offset) const
 	// save categories
 	if( Categories.size() ) {
 		string store;
-		CategoryList2Str(Categories, store);
-		BuildField(data, offset, CFC_CATEGORY, store);
+		Categories.CategoryList2Str(store);
+		BuildField(data, offset, CFC_CATEGORY, ic ? ic->ToBB(store) : store);
 	}
 
 	// save Birthday and Anniversary
@@ -459,6 +489,11 @@ void Contact::Dump(std::ostream &os) const
 		b->type != CFC_INVALID_FIELD;
 		b++ )
 	{
+		// special case: don't dump the raw image data, but
+		// leave that for a special hex dump
+		if( b->type == CFC_IMAGE )
+			continue;
+
 		const std::string *pField = 0;
 		if( b->strMember ) {
 			pField = &(this->*(b->strMember));
@@ -476,7 +511,7 @@ void Contact::Dump(std::ostream &os) const
 
 	if( Categories.size() ) {
 		string display;
-		CategoryList2Str(Categories, display);
+		Categories.CategoryList2Str(display);
 		os << "    Categories          : " << display << "\n";
 	}
 
@@ -497,6 +532,13 @@ void Contact::Dump(std::ostream &os) const
 		os << "        ID: 0x" << setbase(16) << gb->Link << "\n";
 	}
 
+	// print Image in hex dump format, if available
+	if( Image.size() ) {
+		Data image(Image.data(), Image.size());
+		os << "    Photo image:\n";
+		os << image << "\n";
+	}
+
 	// and finally print unknowns
 	os << Unknowns;
 
@@ -522,55 +564,5 @@ void Contact::SplitName(const std::string &full, std::string &first, std::string
 	}
 }
 
-void Contact::CategoryStr2List(const std::string &str,
-			       Barry::CategoryList &list)
-{
-	// start fresh
-	list.clear();
-
-	if( !str.size() )
-		return;
-
-	// parse the comma-delimited string to a list, stripping away
-	// any white space around each category name
-	string::size_type start = 0, end = 0, delim = str.find(',', start);
-	while( start != string::npos ) {
-		if( delim == string::npos )
-			end = str.size() - 1;
-		else
-			end = delim - 1;
-
-		// strip surrounding whitespace
-		while( str[start] == ' ' )
-			start++;
-		while( end && str[end] == ' ' )
-			end--;
-
-		if( start <= end ) {
-			string token = str.substr(start, end-start+1);
-			list.push_back(token);
-		}
-
-		// next
-		start = delim;
-		if( start != string::npos )
-			start++;
-		delim = str.find(',', start);
-	}
-}
-
-void Contact::CategoryList2Str(const Barry::CategoryList &list,
-			       std::string &str)
-{
-	str.clear();
-
-	Barry::CategoryList::const_iterator i = list.begin();
-	for( ; i != list.end(); ++i ) {
-		if( str.size() )
-			str += ", ";
-		str += *i;
-	}
-}
-
 } // namespace Barry
 
