@@ -9,7 +9,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2008, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -36,6 +36,7 @@
 #include <iomanip>
 #include <time.h>
 #include <string.h>
+#include <stdio.h>			// for sscanf()
 #include <stdexcept>
 
 #define __DEBUG_MODE__
@@ -64,6 +65,11 @@ void BuildField1900(Data &data, size_t &size, uint8_t type, time_t t)
 }
 
 void BuildField(Data &data, size_t &size, uint8_t type, char c)
+{
+	BuildField(data, size, type, (uint8_t)c);
+}
+
+void BuildField(Data &data, size_t &size, uint8_t type, uint8_t c)
 {
 	size_t strsize = 1;
 	size_t fieldsize = COMMON_FIELD_HEADER_SIZE + strsize;
@@ -93,6 +99,22 @@ void BuildField(Data &data, size_t &size, uint8_t type, uint16_t value)
 	size += fieldsize;
 }
 
+void BuildField(Data &data, size_t &size, uint8_t type, uint32_t value)
+{
+	size_t strsize = 4;
+	size_t fieldsize = COMMON_FIELD_HEADER_SIZE + strsize;
+	unsigned char *pd = data.GetBuffer(size + fieldsize) + size;
+	CommonField *field = (CommonField *) pd;
+
+	field->size = htobl(strsize);
+	field->type = type;
+
+	uint32_t store = htobs(value);
+	memcpy(field->u.raw, &store, strsize);
+
+	size += fieldsize;
+}
+
 void BuildField(Data &data, size_t &size, uint8_t type, const std::string &str)
 {
 	// include null terminator
@@ -505,7 +527,16 @@ std::ostream& operator<< (std::ostream &os, const std::vector<UnknownField> &unk
 // EmailAddress class
 
 std::ostream& operator<<(std::ostream &os, const EmailAddress &msga) {
-	os << msga.Name.c_str() << " <" << msga.Email.c_str() << ">";
+	os << msga.Name << " <" << msga.Email << ">";
+	return os;
+}
+
+std::ostream& operator<<(std::ostream &os, const EmailAddressList &elist) {
+	for( EmailAddressList::const_iterator i = elist.begin(); i != elist.end(); ++i ) {
+		if( i != elist.begin() )
+			os << ", ";
+		os << *i;
+	}
 	return os;
 }
 
@@ -543,7 +574,7 @@ std::string PostalAddress::GetLabel() const
 		address += "\n";
 	if( PostalCode.size() )
 		address += PostalCode;
-	
+
 	return address;
 }
 
@@ -589,9 +620,10 @@ void Date::ToTm(struct tm *timep) const
 std::string Date::ToYYYYMMDD() const
 {
 	std::ostringstream oss;
-	oss	<< setw(4) << Year
-		<< setw(2) << Month + 1
-		<< setw(2) << Day;
+	// setfill and setw not sticky.
+	oss	<< setw(4) << setfill('0') << Year
+		<< setw(2) << setfill('0') << Month + 1
+		<< setw(2) << setfill('0') << Day;
 	return oss.str();
 }
 
@@ -604,9 +636,10 @@ std::string Date::ToYYYYMMDD() const
 std::string Date::ToBBString() const
 {
 	std::ostringstream oss;
-	oss	<< setw(2) << Day
-		<< Month + 1
-		<< Year;
+	// setw() ain't 'sticky'!
+	oss	<< setw(2) << setfill('0') << Day << '/'
+		<< setw(2) << setfill('0') << Month + 1 << '/'
+		<< setw(2) << setfill('0') << Year;
 	return oss.str();
 }
 
@@ -651,6 +684,62 @@ std::ostream& operator<<(std::ostream &os, const Date &date)
 }
 
 
+///////////////////////////////////////////////////////////////////////////////
+// CategoryList class
+
+/// Parses the given comma delimited category string into
+/// this CategoryList object, appending each token to the vector.
+/// Will clear vector beforehand.
+void CategoryList::CategoryStr2List(const std::string &str)
+{
+	// start fresh
+	clear();
+
+	if( !str.size() )
+		return;
+
+	// parse the comma-delimited string to a list, stripping away
+	// any white space around each category name
+	string::size_type start = 0, end = 0, delim = str.find(',', start);
+	while( start != string::npos ) {
+		if( delim == string::npos )
+			end = str.size() - 1;
+		else
+			end = delim - 1;
+
+		// strip surrounding whitespace
+		while( str[start] == ' ' )
+			start++;
+		while( end && str[end] == ' ' )
+			end--;
+
+		if( start <= end ) {
+			string token = str.substr(start, end-start+1);
+			push_back(token);
+		}
+
+		// next
+		start = delim;
+		if( start != string::npos )
+			start++;
+		delim = str.find(',', start);
+	}
+}
+
+/// Turns the current vectory into a comma delimited category
+/// string suitable for use in Calendar, Task, and Memo protocol values.
+void CategoryList::CategoryList2Str(std::string &str) const
+{
+	str.clear();
+
+	Barry::CategoryList::const_iterator i = begin();
+	for( ; i != end(); ++i ) {
+		if( str.size() )
+			str += ", ";
+		str += *i;
+	}
+}
+
 
 } // namespace Barry
 
