@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2008, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -51,7 +51,9 @@ SocketZero::SocketZero(	SocketRoutingQueue &queue,
 	m_sequenceId(0),
 	m_halfOpen(false),
 	m_challengeSeed(0),
-	m_remainingTries(0)
+	m_remainingTries(0),
+	m_hideSequencePacket(true),
+	m_resetOnClose(false)
 {
 }
 
@@ -66,7 +68,9 @@ SocketZero::SocketZero(	Device &dev,
 	m_sequenceId(0),
 	m_halfOpen(false),
 	m_challengeSeed(0),
-	m_remainingTries(0)
+	m_remainingTries(0),
+	m_hideSequencePacket(true),
+	m_resetOnClose(false)
 {
 }
 
@@ -186,7 +190,7 @@ void SocketZero::CheckSequence(uint16_t socket, const Data &seq)
 			}
 			else {
 				dout("Bad sequence on socket 0: expected: "
-					<< msequenceId
+					<< m_sequenceId
 					<< ". Packet sequence: " << sequenceId);
 			}
 		}
@@ -216,7 +220,7 @@ void SocketZero::SendOpen(uint16_t socket, Data &receive)
 	}
 
 	// check sequence ID
-	Protocol::CheckSize(receive);
+	Protocol::CheckSize(receive, SB_PACKET_HEADER_SIZE);
 	if( IS_COMMAND(receive, SB_COMMAND_SEQUENCE_HANDSHAKE) ) {
 		CheckSequence(0, receive);
 
@@ -274,7 +278,7 @@ void SocketZero::SendPasswordHash(uint16_t socket, const char *password, Data &r
 	send.Zap();
 
 	// check sequence ID
-	Protocol::CheckSize(receive);
+	Protocol::CheckSize(receive, SB_PACKET_HEADER_SIZE);
 	if( IS_COMMAND(receive, SB_COMMAND_SEQUENCE_HANDSHAKE) ) {
 		CheckSequence(0, receive);
 
@@ -339,8 +343,17 @@ void SocketZero::RawReceive(Data &receive, int timeout)
 ///
 bool SocketZero::SequencePacket(const Data &data)
 {
+	// Begin -- Test quiet durty :(
+	if (m_hideSequencePacket == false) {
+		return false;
+	}
+	// End -- Test quiet durty :(
+
 	if( data.GetSize() >= MIN_PACKET_SIZE ) {
-		if( IS_COMMAND(data, SB_COMMAND_SEQUENCE_HANDSHAKE) ) {
+		MAKE_PACKET(rpack, data);
+		if( rpack->socket == 0 &&
+		    rpack->command == SB_COMMAND_SEQUENCE_HANDSHAKE )
+		{
 			CheckSequence(0, data);
 			return true;
 		}
@@ -393,6 +406,11 @@ void SocketZero::Send(Barry::Packet &packet, int timeout)
 	Send(packet.m_send, packet.m_receive, timeout);
 }
 
+void SocketZero::Receive(Data &receive, int timeout)
+{
+	RawReceive(receive, timeout);
+}
+
 
 //
 // Open
@@ -531,7 +549,7 @@ void SocketZero::Close(Socket &socket)
 	}
 
 	// starting fresh, reset sequence ID
-	Protocol::CheckSize(response);
+	Protocol::CheckSize(response, SB_PACKET_HEADER_SIZE);
 	if( IS_COMMAND(response, SB_COMMAND_SEQUENCE_HANDSHAKE) ) {
 		CheckSequence(0, response);
 
@@ -549,7 +567,19 @@ void SocketZero::Close(Socket &socket)
 		socket.ForceClosed();
 
 		eout("Packet:\n" << response);
-		throw Error("Socket: Bad CLOSED packet in Close");
+		throw BadPacket(rpack->command, "Socket: Bad CLOSED packet in Close");
+	}
+
+	if( m_resetOnClose ) {
+		Data send, receive;
+		ZeroPacket reset_packet(send, receive);
+		reset_packet.Reset();
+
+		Send(reset_packet);
+		if( reset_packet.CommandResponse() != SB_COMMAND_RESET_REPLY ) {
+			throw BadPacket(reset_packet.CommandResponse(),
+				"Socket: Missing RESET_REPLY in Close");
+		}
 	}
 
 //	// and finally, there always seems to be an extra read of
@@ -676,29 +706,103 @@ void Socket::Receive(Data &receive, int timeout)
 	}
 }
 
+
+// sends the send packet down to the device
+// Blocks until response received or timed out in Usb::Device
+void Socket::PacketData(Data &send, Data &receive, int timeout)
+{
+	if( ( send.GetSize() < MIN_PACKET_DATA_SIZE ) ||
+		( send.GetSize() > MAX_PACKET_DATA_SIZE ) ) {
+		// we don't do that around here
+		throw std::logic_error("Socket: unknown send data in PacketData()");
+	}
+
+	Data &inFrag = receive;
+	receive.Zap();
+
+	// send non-fragmented
+	Send(send, inFrag, timeout);
+
+	bool done = false;
+	int blankCount = 0;
+
+	while( !done ) {
+		// check the packet's validity
+		if( inFrag.GetSize() > 0 ) {
+			MAKE_PACKET(rpack, inFrag);
+
+			blankCount = 0;
+
+			Protocol::CheckSize(inFrag, SB_PACKET_HEADER_SIZE);
+
+			switch( rpack->command )
+			{
+			case SB_COMMAND_SEQUENCE_HANDSHAKE:
+				CheckSequence(inFrag);
+				if (!m_zero->IsSequencePacketHidden())
+					done = true;
+				break;
+
+			case SB_COMMAND_JL_READY:
+			case SB_COMMAND_JL_ACK:
+			case SB_COMMAND_JL_HELLO_ACK:
+			case SB_COMMAND_JL_RESET_REQUIRED:
+				done = true;
+				break;
+
+			case SB_COMMAND_JL_GET_DATA_ENTRY:	// This response means that the next packet is the stream
+				done = true;
+				break;
+
+			case SB_DATA_JL_INVALID:
+				throw BadPacket(rpack->command, "file is not a valid Java code file");
+				break;
+
+			case SB_COMMAND_JL_NOT_SUPPORTED:
+				throw BadPacket(rpack->command, "device does not support requested command");
+				break;
+
+			default:
+				// unknown packet, pass it up to the
+				// next higher code layer
+				done = true;
+				break;
+			}
+		}
+		else {
+			blankCount++;
+			//std::cerr << "Blank! " << blankCount << std::endl;
+			if( blankCount == 10 ) {
+				// only ask for more data on stalled sockets
+				// for so long
+				throw Error("Socket: 10 blank packets received");
+			}
+		}
+
+		if( !done ) {
+			// not done yet, ask for another read
+			Receive(inFrag);
+		}
+	}
+}
+
 // sends the send packet down to the device, fragmenting if
 // necessary, and returns the response in receive, defragmenting
 // if needed
 // Blocks until response received or timed out in Usb::Device
+//
+// This is primarily for Desktop Database packets... Javaloader
+// packets use PacketData().
+//
 void Socket::Packet(Data &send, Data &receive, int timeout)
 {
-/*
-// FIXME - this might be a good idea someday, or perhaps provide a wrapper
-// function that forces the socket number to the correct current value,
-// but putting it here means a copy on every packet.
-
-	// force socket to our socket
-	Data send = sendorig;
-	Barry::Protocol::Packet *sspack = (Barry::Protocol::Packet *)send.GetBuffer(2);
-	sspack->socket = htobs(GetSocket());
-*/
-
 	MAKE_PACKET(spack, send);
 	if( send.GetSize() < MIN_PACKET_SIZE ||
 	    (spack->command != SB_COMMAND_DB_DATA &&
 	     spack->command != SB_COMMAND_DB_DONE) )
 	{
 		// we don't do that around here
+		eout("unknown send data in Packet(): " << send);
 		throw std::logic_error("Socket: unknown send data in Packet()");
 	}
 
@@ -714,36 +818,56 @@ void Socket::Packet(Data &send, Data &receive, int timeout)
 		unsigned int offset = 0;
 		Data outFrag;
 
+		// You haven't to sequence packet while the whole packet isn't sent
+		//  a) No sequence received packet
+		//  b) 1°) Sent framgment 1/N
+		//     2°) Sent framgment 2/N
+		//         ...
+		//     N°) Before sent fragment N/N, I enable the sequence packet process.
+		//         Sent framgment N/N
+		HideSequencePacket(false);
+
 		do {
 			offset = SocketZero::MakeNextFragment(send, outFrag, offset);
+
+			// Is last packet ?
+			MAKE_PACKET(spack, outFrag);
+
+			if (spack->command != SB_COMMAND_DB_FRAGMENTED)
+				HideSequencePacket(true);
+
 			Send(outFrag, inFrag, timeout);
 
-			MAKE_PACKET(rpack, inFrag);
 			// only process sequence handshakes... once we
 			// get to the last fragment, we fall through to normal
 			// processing below
-			if( offset && inFrag.GetSize() > 0 ) {
+			if (spack->command != SB_COMMAND_DB_FRAGMENTED) {
+				MAKE_PACKET(rpack, inFrag);
 
-				Protocol::CheckSize(inFrag);
+				if( offset && inFrag.GetSize() > 0 ) {
+					Protocol::CheckSize(inFrag, SB_PACKET_HEADER_SIZE);
 
-				switch( rpack->command )
-				{
-				case SB_COMMAND_SEQUENCE_HANDSHAKE:
-					CheckSequence(inFrag);
-					break;
+					switch( rpack->command )
+					{
+					case SB_COMMAND_SEQUENCE_HANDSHAKE:
+						CheckSequence(inFrag);
+						break;
 
-				default: {
-					std::ostringstream oss;
-					oss << "Socket: unhandled packet in Packet() (send): 0x" << std::hex << (unsigned int)rpack->command;
-					eout(oss.str());
-					throw Error(oss.str());
+					default: {
+						std::ostringstream oss;
+						oss << "Socket: (send) unhandled packet in Packet(): 0x" << std::hex << (unsigned int)rpack->command;
+						eout(oss.str());
+						throw Error(oss.str());
+						}
+						break;
 					}
-					break;
 				}
 			}
 
-
 		} while( offset > 0 );
+
+		// To be sure that it's clean...
+		HideSequencePacket(true);
 	}
 
 	bool done = false, frag = false;
@@ -755,7 +879,7 @@ void Socket::Packet(Data &send, Data &receive, int timeout)
 		if( inFrag.GetSize() > 0 ) {
 			blankCount = 0;
 
-			Protocol::CheckSize(inFrag);
+			Protocol::CheckSize(inFrag, SB_PACKET_HEADER_SIZE);
 
 			switch( rpack->command )
 			{
@@ -785,7 +909,7 @@ void Socket::Packet(Data &send, Data &receive, int timeout)
 
 			default: {
 				std::ostringstream oss;
-				oss << "Socket: unhandled packet in Packet() (read): 0x" << std::hex << (unsigned int)rpack->command;
+				oss << "Socket: (read) unhandled packet in Packet(): 0x" << std::hex << (unsigned int)rpack->command;
 				eout(oss.str());
 				throw Error(oss.str());
 				}
@@ -814,6 +938,19 @@ void Socket::Packet(Barry::Packet &packet, int timeout)
 	Packet(packet.m_send, packet.m_receive, timeout);
 }
 
+void Socket::Packet(Barry::JLPacket &packet, int timeout)
+{
+	if( packet.HasData() ) {
+		HideSequencePacket(false);
+		PacketData(packet.m_cmd, packet.m_receive, timeout);
+		HideSequencePacket(true);
+		PacketData(packet.m_data, packet.m_receive, timeout);
+	}
+	else {
+		PacketData(packet.m_cmd, packet.m_receive, timeout);
+	}
+}
+
 void Socket::NextRecord(Data &receive)
 {
 	Barry::Protocol::Packet packet;
