@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2008, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@ void Usage()
 
    cerr
    << "btool - Command line USB Blackberry Test Tool\n"
-   << "        Copyright 2005-2008, Net Direct Inc. (http://www.netdirect.ca/)\n"
+   << "        Copyright 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)\n"
    << "        Using: " << Version << "\n"
    << "        Compiled "
 #ifdef __BARRY_BOOST_MODE__
@@ -53,6 +53,8 @@ void Usage()
    << "   -B bus    Specify which USB bus to search on\n"
    << "   -N dev    Specify which system device, using system specific string\n"
    << "\n"
+   << "   -a db     Clear database 'db' FROM device\n"
+   << "             Can be used multiple times to clear more than one DB\n"
    << "   -c dn     Convert address book database to LDIF format, using the\n"
    << "             specified baseDN\n"
    << "   -C dnattr LDIF attribute name to use when building the FQDN\n"
@@ -68,6 +70,8 @@ void Usage()
    << "   -f file   Filename to save or load handheld data to/from\n"
 #endif
    << "   -h        This help\n"
+   << "   -i cs     International charset for string conversions\n"
+   << "             Valid values here are available with 'iconv --list'\n"
    << "   -l        List devices\n"
    << "   -L        List Contact field names\n"
    << "   -m        Map LDIF name to Contact field / Unmap LDIF name\n"
@@ -206,17 +210,24 @@ class DataDumpParser : public Barry::Parser
 	uint32_t m_id;
 
 public:
+	virtual void Clear() {}
+
 	virtual void SetIds(uint8_t RecType, uint32_t UniqueId)
 	{
 		m_id = UniqueId;
 	}
 
-	virtual void ParseFields(const Barry::Data &data, size_t &offset)
+	virtual void ParseHeader(const Data &, size_t &) {}
+
+	virtual void ParseFields(const Barry::Data &data, size_t &offset,
+				const IConverter *ic)
 	{
 		std::cout << "Raw record dump for record: "
 			<< std::hex << m_id << std::endl;
 		std::cout << data << std::endl;
 	}
+
+	virtual void Store() {}
 };
 
 auto_ptr<Parser> GetParser(const string &name, const string &filename, bool null_parser)
@@ -241,6 +252,11 @@ auto_ptr<Parser> GetParser(const string &name, const string &filename, bool null
 			new RecordParser<Calendar, Store<Calendar> > (
 				new Store<Calendar>(filename, false)));
 	}
+	else if( name == CallLog::GetDBName() ) {
+		return auto_ptr<Parser>(
+			new RecordParser<CallLog, Store<CallLog> > (
+				new Store<CallLog>(filename, false)));
+	}
 	else if( name == ServiceBook::GetDBName() ) {
 		return auto_ptr<Parser>(
 			new RecordParser<ServiceBook, Store<ServiceBook> > (
@@ -267,6 +283,11 @@ auto_ptr<Parser> GetParser(const string &name, const string &filename, bool null
 			new RecordParser<SavedMessage, Store<SavedMessage> > (
 				new Store<SavedMessage>(filename, false)));
 	}
+	else if( name == Sms::GetDBName() ) {
+		return auto_ptr<Parser>(
+			new RecordParser<Sms, Store<Sms> > (
+				new Store<Sms>(filename, false)));
+	}
 	else if( name == Folder::GetDBName() ) {
 		return auto_ptr<Parser>(
 			new RecordParser<Folder, Store<Folder> > (
@@ -296,6 +317,16 @@ auto_ptr<Builder> GetBuilder(const string &name, const string &filename)
 			new RecordBuilder<Calendar, Store<Calendar> > (
 				new Store<Calendar>(filename, true)));
 	}
+	else if( name == Memo::GetDBName() ) {
+		return auto_ptr<Builder>(
+			new RecordBuilder<Memo, Store<Memo> > (
+				new Store<Memo>(filename, true)));
+	}
+	else if( name == Task::GetDBName() ) {
+		return auto_ptr<Builder>(
+			new RecordBuilder<Task, Store<Task> > (
+				new Store<Task>(filename, true)));
+	}
 /*
 	else if( name == "Messages" ) {
 		return auto_ptr<Parser>(
@@ -307,16 +338,6 @@ auto_ptr<Builder> GetBuilder(const string &name, const string &filename)
 			new RecordParser<ServiceBook, Store<ServiceBook> > (
 				new Store<ServiceBook>(filename, true)));
 	}
-	else if( name == "Memos" ) {
-		return auto_ptr<Parser>(
-			new RecordParser<Memo, Store<Memo> > (
-				new Store<Memo>(filename, true)));
-	}
-	else if( name == "Tasks" ) {
-		return auto_ptr<Parser>(
-			new RecordParser<Task, Store<Task> > (
-				new Store<Task>(filename, true)));
-	}
 */
 	else {
 		throw std::runtime_error("No Builder available for database");
@@ -329,17 +350,21 @@ void ShowParsers()
 	<< "   Address Book\n"
 	<< "   Messages\n"
 	<< "   Calendar\n"
+	<< "   Phone Call Logs\n"
 	<< "   Service Book\n"
 	<< "   Memos\n"
 	<< "   Tasks\n"
 	<< "   PIN Messages\n"
 	<< "   Saved Email Messages\n"
+	<< "   SMS Messages\n"
 	<< "   Folders\n"
 	<< "   Time Zones (read only)\n"
 	<< "\n"
 	<< "Supported Database builders:\n"
 	<< "   Address Book\n"
 	<< "   Calendar\n"
+	<< "   Memo\n"
+	<< "   Task\n"
 	<< endl;
 }
 
@@ -430,24 +455,31 @@ int main(int argc, char *argv[])
 			epp_override = false,
 			threaded_sockets = true,
 			record_state = false,
+			clear_database = false,
 			null_parser = false;
 		string ldifBaseDN, ldifDnAttr;
 		string filename;
 		string password;
 		string busname;
 		string devname;
-		vector<string> dbNames, saveDbNames, mapCommands;
+		string iconvCharset;
+		vector<string> dbNames, saveDbNames, mapCommands, clearDbNames;
 		vector<StateTableCommand> stCommands;
 		Usb::EndpointPair epOverride;
 
 		// process command line options
 		for(;;) {
-			int cmd = getopt(argc, argv, "B:c:C:d:D:e:f:hlLm:MnN:p:P:r:R:Ss:tT:vXzZ");
+			int cmd = getopt(argc, argv, "a:B:c:C:d:D:e:f:hi:lLm:MnN:p:P:r:R:Ss:tT:vXzZ");
 			if( cmd == -1 )
 				break;
 
 			switch( cmd )
 			{
+			case 'a':	// Clear Database
+				clear_database = true;
+				clearDbNames.push_back(string(optarg));
+				break;
+
 			case 'B':	// busname
 				busname = optarg;
 				break;
@@ -487,6 +519,11 @@ int main(int argc, char *argv[])
 				return 1;
 #endif
 				break;
+
+			case 'i':	// international charset (iconv)
+				iconvCharset = optarg;
+				break;
+
 			case 'l':	// list only
 				list_only = true;
 				break;
@@ -573,6 +610,12 @@ int main(int argc, char *argv[])
 		// anything else.
 		Barry::Init(data_dump);
 
+		// Create an IConverter object if needed
+		auto_ptr<IConverter> ic;
+		if( iconvCharset.size() ) {
+			ic.reset( new IConverter(iconvCharset.c_str(), true) );
+		}
+
 		// LDIF class... only needed if ldif output turned on
 		ContactLdif ldif(ldifBaseDN);
 		DoMapping(ldif, mapCommands);
@@ -585,7 +628,8 @@ int main(int argc, char *argv[])
 		// Probe the USB bus for Blackberry devices and display.
 		// If user has specified a PIN, search for it in the
 		// available device list here as well
-		Barry::Probe probe(busname.c_str(), devname.c_str());
+		Barry::Probe probe(busname.c_str(), devname.c_str(),
+			epp_override ? &epOverride : 0);
 		int activeDevice = -1;
 
 		// show any errors during probe first
@@ -681,7 +725,7 @@ int main(int argc, char *argv[])
 		}
 
 		Barry::Controller &con = *pcon;
-		Barry::Mode::Desktop desktop(con);
+		Barry::Mode::Desktop desktop(con, *ic);
 
 		//
 		// execute each mode that was turned on
@@ -770,6 +814,26 @@ int main(int argc, char *argv[])
 			return 0;
 		}
 
+		// Clear databases
+		if (clear_database) {
+			if( clearDbNames.size() == 0 ) {
+				cout << "No db names to clear" << endl;
+				return 1;
+			}
+
+			vector<string>::iterator b = clearDbNames.begin();
+
+			desktop.Open(password.c_str());
+
+			for( ; b != clearDbNames.end(); b++ ) {
+				unsigned int id = desktop.GetDBID(*b);
+				cout << "Clearing all recordss from " << (*b) << "..." << endl;
+				desktop.ClearDatabase(id);
+			}
+
+			return 0;
+		}
+
 		// Dump contents of selected databases to stdout, or
 		// to file if specified.
 		// This is retrieving data from the Blackberry.
